// SASAX-RSS: RSS(RDF Site Summary) parser by SASAX
// Copyright (C) 2004 FUJIWARA Katsunori(foozy@lares.dti.ne.jp)
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

package jp.ne.dti.lares.foozy.sasax.rss;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;

import jp.ne.dti.lares.foozy.sasax.LooseDateTimeElement;
import jp.ne.dti.lares.foozy.sasax.ElementDrivenHandler;

/**
 * Put information into <code>Channel</code> from parsing result.
 *
 * <p>This class works as a kind of deserializer,
 * which deserializes {@link Channel} from RSS document
 * by SASAX/SASAX-RSS.</p>
 *
 * <ol>
 * <li>get <code>InputStream</code> for RSS document from specified location
 * <li>set <code>Notification</code>s to corresponded element
 * <li>parse RSS document, and it causes deserialization
 * </ol>
 *
 * @author FUJIWARA Katsunori &lt;foozy@lares.dti.ne.jp&gt;(code/doc)
 */
public class ChannelFactory
{
    /**
     * Create <code>Channel</code> from RSS document.
     *
     * @param location of RSS document
     */
    static
    public Channel create(String location)
        throws IOException, //
               SAXException, //
               ParserConfigurationException //
    {
        //
        // Get "InputStream" for RSS document from specified location
        //
        URL url = new URL(location);
        URLConnection connection = url.openConnection();
        connection.connect();
        String type = connection.getContentType();
        String encoding = connection.getContentEncoding();
        InputStream iStream = connection.getInputStream();

        Channel channel = new Channel();
        RSSRootElement root = new RSSRootElement(null);

        //
        // Set "Notification"s to corresponded element
        //
        root.channel.addNotification(new ChannelNotification(channel));
        root.channel.rdfSeq.li.addNotification(new RDFLiNotification(channel));
        root.item.addNotification(new ItemNotification(channel));
        { // >>>> DC:DATE
            LooseDateTimeElement dcDate =
            new LooseDateTimeElement(root.item,
                                     "http://purl.org/dc/elements/1.1/",
                                     "date",
                                     null // TimeZone
                                     );
            root.item.addOptionalItem(ValueID.DC_DATE, dcDate);
        } // <<<< DC:DATE

        //
        // Parse RSS document, and it causes deserialization
        //
        InputStreamReader reader = null;
        try{
            reader =
            (null != encoding
             ? new InputStreamReader(iStream, encoding)
             : new InputStreamReader(iStream, "UTF8"));

            ElementDrivenHandler handler = new ElementDrivenHandler(root);
            handler.parse(reader);
        }
        finally{
            if(null != reader){ reader.close(); }
        }

        return channel;
    }
}
