// SASAX-RSS: RSS(RDF Site Summary) parser by SASAX
// Copyright (C) 2004 FUJIWARA Katsunori(foozy@lares.dti.ne.jp)
// 
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

package jp.ne.dti.lares.foozy.sasax.rss;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import jp.ne.dti.lares.foozy.sasax.Element;
import jp.ne.dti.lares.foozy.sasax.Notification;
import jp.ne.dti.lares.foozy.sasax.ParseContext;

/**
 * Listener to set information up from parse result of "channel" element.
 *
 * <p>This expects to be added to <code>ChannelElement</code>.</p>
 *
 * @author FUJIWARA Katsunori &lt;foozy@lares.dti.ne.jp&gt;(code/doc)
 */
public class ChannelNotification
    implements Notification //
{
    final
    private Channel channel_;

    ///////////////////////////////////////////////////////////////////////////

    /**
     * Constructor.
     *
     * @param channel into which information is stored
     */
    public ChannelNotification(Channel channel){
        channel_ = channel;
    }

    ///////////////////////////////////////////////////////////////////////////
    // Implementation of interface Notification

    /**
     * Invoked when target element is "started".
     *
     * <p>This store value(s) shown below into channel,
     * which is specified at construction time.</p>
     *
     * <ul>
     * <li>"rdf:about" attribute value by {@link ValueID#RDF_ABOUT}
     * </ul>
     *
     * @param element which is started
     * @param context of SAX document parsing.
     * @param attrs given by "startElement" SAX event.
     */
    public void elementStarted(Element element,
                               ParseContext context,
                               Attributes attributes)
        throws SAXException //
    {
        String about = attributes.getValue(RDF.NS_URI, RDF.ABOUT);
        channel_.setValue(ValueID.RDF_ABOUT, about);
    }

    /**
     * Invoked when target element is "ended".
     *
     * <p>This store value(s) shown below into channel,
     * which is specified at construction time.</p>
     *
     * <ul>
     * <li>"title" element value by {@link ValueID#TITLE}
     * <li>"link" element value by {@link ValueID#LINK}
     * <li>"description" element value by {@link ValueID#DESCRIPTION}
     * </ul>
     *
     * @param element which is ended
     * @param context of SAX document parsing.
     */
    public void elementEnded(Element element,
                             ParseContext context)
        throws SAXException //
    {
        ChannelElement channel = (ChannelElement)element;
        channel_.setValue(ValueID.TITLE, channel.title.getString(true));
        channel_.setValue(ValueID.LINK, channel.link.getString(true));
        channel_.setValue(ValueID.DESCRIPTION,
                          channel.description.getString(true));
    }
}
